#include "CreatorMap.h"
#include "OEMSolver.h"
#include "ITL_Solver.h"
#include "ITL_Preconditioner.h"
#include "NonLinSolver.h"
#include "MatrixVector.h"
#include "SystemVector.h"
#include "Estimator.h"
#include "RecoveryEstimator.h"
#include "ResidualEstimator.h"
#include "LeafData.h"
#include "SurfaceRegion_ED.h"
#include "ElementRegion_ED.h"
#include "PartitionElementData.h"
#include "DOFMatrix.h"
#include "UmfPackSolver.h"

namespace AMDiS {

  template<>
  void CreatorMap<OEMSolver>::addDefaultCreators()
  {
    OEMSolverCreator *creator;

    creator = new CGSolver::Creator;
    addCreator("cg", creator);

    creator = new CGSSolver::Creator;
    addCreator("cgs", creator);

    creator = new BiCGSolver::Creator;
    addCreator("bicg", creator);

    creator = new BiCGStabSolver::Creator;
    addCreator("bicgstab", creator);

    creator = new BiCGStab2Solver::Creator;
    addCreator("bicgstab2", creator);

    creator = new BiCGStabEllSolver::Creator;
    addCreator("bicgstab_ell", creator);

    creator = new QMRSolver::Creator;
    addCreator("qmr", creator);

    creator = new TFQMRSolver::Creator;
    addCreator("tfqmr", creator);

    creator = new GMResSolver::Creator;
    addCreator("gmres", creator);

    creator = new IDRsSolver::Creator;
    addCreator("idr_s", creator);

#ifdef HAVE_UMFPACK
    creator = new UmfPackSolver::Creator;
    addCreator("umfpack", creator);
#endif
  }

  template<>
  void CreatorMap<ITL_BasePreconditioner>::addDefaultCreators()
  {
    CreatorInterface<ITL_BasePreconditioner> *creator;

    creator =  new DiagonalPreconditioner::Creator;
    addCreator("diag", creator);

    creator = new ILUPreconditioner::Creator;
    addCreator("ilu", creator);

    creator = new ICPreconditioner::Creator;
    addCreator("ic", creator);

    creator =  new IdentityPreconditioner::Creator;
    addCreator("no", creator);
  }

#if 0
  template<>
  void CreatorMap<NonLinSolver<DOFVector<double> > >::addDefaultCreators()
  {
    NonLinSolverCreator<DOFVector<double> > *creator;

    creator = new Newton<DOFVector<double> >::Creator;
    addCreator("newton", creator);

    creator = new NewtonS<DOFVector<double> >::Creator;
    addCreator("newton_fs", creator);
  }
#endif


#if 0
  template<>
  void CreatorMap<NonLinSolver<SystemVector> >::addDefaultCreators()
  {
    NonLinSolverCreator<SystemVector> *creator;

    creator = new Newton<SystemVector>::Creator;
    addCreator("newton", creator);

    creator = new NewtonS<SystemVector>::Creator;
    addCreator("newton_fs", creator);
  }
#endif

  template<>
  void CreatorMap<Estimator>::addDefaultCreators()
  {
    EstimatorCreator *creator;

    creator = new ResidualEstimator::Creator;
    addCreator("residual", creator);

    creator = new RecoveryEstimator::Creator;
    addCreator("recovery", creator);
  }

  template<>
  void CreatorMap<ElementData>::addDefaultCreators()
  {
    CreatorInterface<ElementData> *creator;

    creator = new LeafDataEstimatable::Creator;
    addCreator("LeafDataEstimatable", creator);

    creator = new LeafDataEstimatableVec::Creator;
    addCreator("LeafDataEstimatableVec", creator);

    creator = new LeafDataCoarsenable::Creator;
    addCreator("LeafDataCoarsenable", creator);

    creator = new LeafDataCoarsenableVec::Creator;
    addCreator("LeafDataCoarsenableVec", creator);

    creator = new LeafDataPeriodic::Creator;
    addCreator("LeafDataPeriodic", creator);

    creator = new SurfaceRegion_ED::Creator;
    addCreator("SurfaceRegion_ED", creator);

    creator = new ElementRegion_ED::Creator;
    addCreator("ElementRegion_ED", creator);

    creator = new PartitionElementData::Creator;
    addCreator("PartitionElementData", creator);
  }

}
