// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  TU Dresden                                                            ==
// ==                                                                        ==
// ==  Institut fr Wissenschaftliches Rechnen                               ==
// ==  Zellescher Weg 12-14                                                  ==
// ==  01069 Dresden                                                         ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  https://gforge.zih.tu-dresden.de/projects/amdis/                      ==
// ==                                                                        ==
// ============================================================================

/** \file ParallelDomainVec.h */

#ifndef AMDIS_PARALLELDOMAINVEC_H
#define AMDIS_PARALLELDOMAINVEC_H

#include "ParallelDomainBase.h"
#include "ProblemVec.h"

namespace AMDiS {

  class ParallelDomainVec : public ParallelDomainBase
  {
  public:
    ParallelDomainVec(ProblemVec *problem,
		      ProblemInstatVec *problemInstat);

    void initParallelization(AdaptInfo *adaptInfo);

    virtual void solveInitialProblem(AdaptInfo *adaptInfo)
    {
      ParallelDomainBase::solveInitialProblem(adaptInfo);

      synchVectors(*(probVec->getSolution()));
    }

    // Writes all data of this object to an output stream.
    virtual void serialize(std::ostream &out)
    {
      probVec->serialize(out);
      ParallelDomainBase::serialize(out);
    }

    // Reads the object data from an input stream.
    virtual void deserialize(std::istream &in)
    {
      probVec->deserialize(in);
      ParallelDomainBase::deserialize(in);
    }

  protected:
    /// Starts the solution of the linear system using Petsc.
    void solve();

  protected:
    /// Pointer to the stationary problem.
    ProblemVec *probVec;
  };

}

#endif // AMDIS_PARALLELDOMAINVEC_H
